from __future__ import absolute_import

import datetime

import decimal
from importlib import import_module
from collections import defaultdict
from types import GeneratorType

from . import generators as g, fakers as f, mix_types as t
from . import six


DEFAULT = object()
RANDOM = object()
FAKE = object()
SELECT = object()


class Field(object):
    """ Store field imformation.
    """
    is_relation = False

    def __init__(self, scheme, name):
        self.scheme = scheme
        self.name = name


class Relation(Field):
    """ Store relation field imformation.
    """
    is_relation = True

    def __init__(self, scheme, name, params=None):
        super(Relation, self).__init__(scheme, name)
        self.params = params or dict()


class GeneratorMeta(type):
    """ Precache generators.
    """
    def __new__(mcs, name, bases, params):
        generators = dict()
        fakers = dict()
        types = dict()

        for cls in bases:
            if isinstance(cls, GeneratorMeta):
                generators.update(cls.generators)
                fakers.update(cls.fakers)
                types.update(cls.types)

        generators.update(params.get('generators', dict()))
        fakers.update(params.get('fakers', dict()))
        types.update(params.get('types', dict()))

        generators = dict(mcs.flat_keys(generators))
        types = dict(mcs.flat_keys(types))

        params['generators'] = generators
        params['fakers'] = fakers
        params['types'] = types

        return super(GeneratorMeta, mcs).__new__(mcs, name, bases, params)

    @staticmethod
    def flat_keys(d):
        for key, value in d.items():
            if isinstance(key, (tuple, list)):
                for k in key:
                    yield k, value
                continue
            yield key, value


class Generator(six.with_metaclass(GeneratorMeta)):
    """ Make generators for types.
    """
    generators = {
        bool: g.gen_boolean,
        float: g.gen_float,
        int: g.gen_integer,
        str: g.gen_string,
        datetime.date: g.gen_date,
        datetime.datetime: g.gen_datetime,
        datetime.time: g.gen_time,
        decimal.Decimal: g.gen_decimal,
        t.BigInteger: g.gen_big_integer,
        t.EmailString: f.gen_email,
        t.HostnameString: f.gen_hostname,
        t.IP4String: f.gen_ip4,
        t.NullOrBoolean: g.gen_null_or_boolean,
        t.PositiveDecimal: g.gen_positive_decimal,
        t.PositiveInteger: g.gen_positive_integer,
        t.SmallInteger: g.gen_small_integer,
        t.Text: f.gen_lorem,
        None: g.loop(lambda: ''),
    }

    fakers = {
        ('name', str): f.gen_name,
        ('first_name', str): f.gen_firstname,
        ('firstname', str): f.gen_firstname,
        ('last_name', str): f.gen_lastname,
        ('lastname', str): f.gen_lastname,
        ('title', str): f.gen_lorem,
        ('description', str): f.gen_lorem,
        ('content', str): f.gen_lorem,
        ('city', str): f.gen_city,
        ('email', str): f.gen_email,
        ('username', str): f.gen_username,
        ('login', str): f.gen_username,
    }

    types = {}

    @classmethod
    def cls_to_simple(cls, fcls):
        """ Translate class to one of simple base types.
        """
        return cls.types.get(fcls) or (
            fcls if fcls in cls.generators
            else None
        )

    @staticmethod
    def name_to_simple(fname):
        """ Translate name to one of simple base names.
        """
        fname = fname or ''
        return fname.lower().strip()

    @classmethod
    def gen_maker(cls, fcls, fname=None, fake=False):
        """ Generate value based on class and name.
        """
        fcls = cls.cls_to_simple(fcls)
        fname = cls.name_to_simple(fname)

        gen_maker = cls.generators.get(fcls)

        if fname and fake and (fname, fcls) in cls.fakers:
            gen_maker = cls.fakers.get((fname, fcls)) or gen_maker

        return gen_maker


class TypeMixerMeta(type):
    """ Cache mixers by class.
    """
    mixers = dict()

    def __call__(cls, cls_type, mixer=None, generator=None, fake=True):
        cls_type = cls.__load_cls(cls_type)
        key = (mixer, cls_type)
        if not key in cls.mixers:
            cls.mixers[key] = super(TypeMixerMeta, cls).__call__(
                cls_type, mixer=mixer, generator=generator, fake=fake,
            )
        return cls.mixers[key]

    @staticmethod
    def __load_cls(cls_type):
        if isinstance(cls_type, six.string_types):
            mod, cls_type = cls_type.rsplit('.', 1)
            mod = import_module(mod)
            cls_type = getattr(mod, cls_type)
        return cls_type


class TypeMixer(six.with_metaclass(TypeMixerMeta)):

    fake = FAKE
    generator = Generator
    select = SELECT
    random = RANDOM

    def __init__(self, cls, mixer=None, generator=None, fake=True):
        self.cls = cls
        self.mixer = mixer
        self.fake = fake

        if self.mixer:
            self.fake = self.mixer.fake

        self.fields = dict(self.__load_fields())
        self.generator = generator or self.generator
        self.generators = dict()
        self.post_save_values = defaultdict(list)

    def __repr__(self):
        return "<TypeMixer {0}>".format(self.cls)

    def blend(self, **values):
        """
        Generate instance.

        :param **values: Predefined fields
        """
        target = self.cls()
        self.post_save_values = defaultdict(list)

        defaults = dict(**self.fields)

        # Prepare relations
        for key, params in values.items():
            if '__' in key:
                rname, rvalue = key.split('__', 1)
                field = defaults.get(rname)
                if not isinstance(field, Relation):
                    defaults[rname] = Relation(field.scheme, field.name)
                defaults[rname].params.update({rvalue: params})
                continue
            defaults[key] = params

        # Fill fields
        for fname, fvalue in defaults.items():

            if isinstance(fvalue, Relation):
                self.gen_relation(target, fname, fvalue)

            elif isinstance(fvalue, Field):
                self.gen_field(target, fname, fvalue)

            elif fvalue is self.random:
                self.gen_random(target, fname)

            elif fvalue is self.fake:
                self.gen_fake(target, fname)

            elif fvalue is self.select:
                self.gen_select(target, fname)

            else:
                self.set_value(target, fname, fvalue)

        return target

    @staticmethod
    def set_value(target, field_name, field_value):
        """ Set `value` to `target` as `field_name`.
        """
        if callable(field_value):
            field_value = field_value()

        elif isinstance(field_value, GeneratorType):
            field_value = next(field_value)

        setattr(target, field_name, field_value)

    def gen_value(self, target, fname, fcls, fake=None):
        """ Generate values from basic types.
            Set value to target.
        """
        if fake is None:
            fake = self.fake

        gen = self.get_generator(fcls, fname, fake=fake)
        setattr(target, fname, next(gen))

    def gen_field(self, target, field_name, field):
        """
        Generate value by field.

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        :param relation: Instance of :class:`Field`
        """
        self.gen_value(target, field_name, field.scheme)

    def gen_relation(self, target, field_name, relation):
        """
        Generate a related field by `relation`

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        :param relation: Instance of :class:`Relation`

        """
        mixer = TypeMixer(relation.scheme, self.mixer, self.generator)
        setattr(target, field_name, mixer.blend(**relation.params))

    def gen_random(self, target, field_name):
        """
        Generate random value of field with `field_name` for `target`

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        """
        field = self.fields.get(field_name)
        self.gen_value(target, field_name, field.scheme, fake=False)

    gen_select = gen_random

    def gen_fake(self, target, field_name):
        """
        Generate fake value of field with `field_name` for `target`

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        """
        field = self.fields.get(field_name)
        self.gen_value(target, field_name, field.scheme, fake=True)

    def get_generator(self, field_class, field_name=None, fake=None):
        """
        Cache generator for class.

        :param field_class: Class for looking a generator
        :param field_name: Name of field for generation
        :param fake: Generate fake data instead of random data.
        """
        if fake is None:
            fake = self.fake

        key = (field_class, field_name, fake)

        if not key in self.generators:
            self.generators[key] = self.make_generator(
                field_class, field_name, fake)

        return self.generators[key]

    def make_generator(self, field_class, field_name=None, fake=None):
        """
        Make generator for class.

        :param field_class: Class for looking a generator
        :param field_name: Name of field for generation
        :param fake: Generate fake data instead of random data.
        """
        return self.generator.gen_maker(field_class, field_name, fake)()

    def __load_fields(self):
        """ Generator of scheme's fields.
        """
        for fname in dir(self.cls):
            if fname.startswith('_'):
                continue
            yield fname, Field(getattr(self.cls, fname), fname)


class MetaMixer:

    def __init__(self, mixer, count=5):
        self.count = count
        self.mixer = mixer

    def blend(self, scheme, **values):
        result = []
        for _ in range(self.count):
            result.append(
                self.mixer.blend(scheme, **values)
            )
        return result

    def __getattr__(self, name):
        raise AttributeError('Use "cycle" only for "blend"')


class Mixer(object):
    """
    This class is used for integration to one or more applications.

    :param fake: (True) Generate fake data instead of random data.

    ::
        class SomeScheme:
            score = int
            name = str

        mixer = Mixer()
        instance = mixer.blend(SomeScheme)

        print instance.name
        # Some like: 'Mike Douglass'

    """

    # system flags
    fake = FAKE
    select = SELECT
    random = RANDOM

    # generator's controller class
    type_mixer_cls = TypeMixer

    def __init__(self, fake=True, **params):
        """Initialize Mixer instance.

        :param fake: (True) Generate fake data instead of random data.

        """
        self.params = params
        self.fake = fake

    def __repr__(self):
        return "<Mixer [{0}]>".format('fake' if self.fake else 'rand')

    def blend(self, scheme, **values):
        """Generate instance of `scheme`.

        :param scheme: Scheme class for generation
        :param **values: Predefined fields

        ::
            mixer = Mixer()
            mixer.blend(SomeSheme, active=True)

            print scheme.active
            # True

        """
        type_mixer = self.type_mixer_cls(scheme, mixer=self)
        result = type_mixer.blend(**values)
        result = self.post_generate(result, type_mixer)
        for fname, fvalue in type_mixer.post_save_values.items():
            setattr(result, fname, fvalue)
        return result

    @staticmethod
    def post_generate(target, type_mixer):
        return target

    @staticmethod
    def sequence(func):
        if isinstance(func, six.string_types):
            func = func.format

        def gen():
            counter = 0
            while True:
                yield func(counter)
                counter += 1
        return gen()

    def cycle(self, count=5):
        return MetaMixer(self, count)

# lint_ignore=C901,W0622,F0401



import datetime
import sys
import random
import decimal
from functools import wraps

DEFAULT_STRING_LENGTH = 8
DEFAULT_CHARS = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'  # nolint


def loop(get_func):
    """ Make generator from function.
    """
    @wraps(get_func)
    def wrapper(*args, **kwargs):
        while True:
            yield get_func(*args, **kwargs)

    return wrapper


def get_choice(choices=None, **kwargs):
    if choices is None:
        return None
    return random.choice(choices)

gen_choice = loop(get_choice)


def get_choices(choices=None, length=None, **kwargs):
    gen = gen_choice(choices)
    if length is None:
        length = len(choices)
    return tuple(next(gen) for _ in range(length))

gen_choices = loop(get_choices)


def get_date(low=(1900, 1, 1), high=(2020, 12, 31), **kwargs):
    low = datetime.date(*low)
    high = datetime.date(*high)
    delta = high - low
    delta = datetime.timedelta(
        seconds=random.randrange(delta.days * 24 * 60 * 60 + delta.seconds))
    return low + delta

gen_date = loop(get_date)


def get_time(low=(0, 0, 0), high=(23, 59, 59), **kwargs):
    h = random.randint(low[0], high[0])
    m = random.randint(low[1], high[1])
    s = random.randint(low[2], high[2])

    return datetime.time(h, m, s)

gen_time = loop(get_time)


def get_datetime(low=(1900, 1, 1, 0, 0, 0),
                 high=(2020, 12, 31, 23, 59, 59), **kwargs):
    date = get_date(low[:3], high[:3])
    h = random.randint(low[3], high[3])
    ms = random.randint(low[4], high[4])
    s = random.randint(low[5], high[5])

    return datetime.datetime(date.year, date.month, date.day, h, ms, s)

gen_datetime = loop(get_datetime)


def get_integer(low=-2147483647, high=2147483647, **kwargs):
    return random.randint(low, high)

gen_integer = loop(get_integer)


def get_big_integer(**kwargs):
    return get_integer(low=-9223372036854775808, high=9223372036854775807)

gen_big_integer = loop(get_big_integer)


def get_small_integer(**kwargs):
    return get_integer(low=-32768, high=32768)

gen_small_integer = loop(get_small_integer)


def get_positive_integer(high=4294967294, **kwargs):
    return get_integer(low=0, high=high)

gen_positive_integer = loop(get_positive_integer)


def get_small_positive_integer(**kwargs):
    return get_integer(low=0, high=65536)

gen_small_positive_integer = loop(get_small_positive_integer)


def get_float(**kwargs):
    return random.uniform(sys.float_info.min, sys.float_info.max)

gen_float = loop(get_float)


def get_boolean(**kwargs):
    return get_choice((True, False))

gen_boolean = loop(get_boolean)


def get_null_or_boolean(**kwargs):
    return get_choice((True, False, None))

gen_null_or_boolean = loop(get_null_or_boolean)


def get_string(length=DEFAULT_STRING_LENGTH, chars=DEFAULT_CHARS, **kwargs):
    return ''.join(get_choices(chars, length))

gen_string = loop(get_string)


def get_decimal(i=4, d=2, positive=False, **kwargs):
    i = 10 ** i
    d = 10 ** d
    return decimal.Decimal(
        "{0}.{1}".format(
            get_integer(low=0 if positive else (-i + 1), high=i - 1),
            get_positive_integer(high=d - 1)
        )
    )

gen_decimal = loop(get_decimal)


def get_positive_decimal(**kwargs):
    return get_decimal(positive=True)

gen_positive_decimal = loop(get_positive_decimal)



from __future__ import absolute_import

import datetime

import decimal
from sqlalchemy import func
from sqlalchemy.orm.interfaces import MANYTOONE
from sqlalchemy.types import (
    BIGINT, BOOLEAN, BigInteger, Boolean, CHAR, DATE, DATETIME, DECIMAL, Date,
    DateTime, FLOAT, Float, INT, INTEGER, Integer, NCHAR, NVARCHAR, NUMERIC,
    Numeric, SMALLINT, SmallInteger, String, TEXT, TIME, Text, Time, Unicode,
    UnicodeText, VARCHAR)

from .. import mix_types as t
from ..main import (
    Relation, Field,
    TypeMixer as BaseTypeMixer,
    Generator as BaseGenerator,
    Mixer as BaseMixer)


class Generator(BaseGenerator):
    types = {
        (String, VARCHAR, Unicode, NVARCHAR, NCHAR, CHAR): str,
        (Text, UnicodeText, TEXT): t.Text,
        (Boolean, BOOLEAN): bool,
        (Date, DATE): datetime.date,
        (DateTime, DATETIME): datetime.datetime,
        (Time, TIME): datetime.time,
        (DECIMAL, Numeric, NUMERIC): decimal.Decimal,
        (Float, FLOAT): float,
        (Integer, INTEGER, INT): int,
        (BigInteger, BIGINT): t.BigInteger,
        (SmallInteger, SMALLINT): t.SmallInteger,
    }


class TypeMixer(BaseTypeMixer):

    generator = Generator

    def __init__(self, cls, **params):
        """ Init TypeMixer and save the mapper.
        """
        super(TypeMixer, self).__init__(cls, **params)
        self.mapper = self.cls._sa_class_manager.mapper

    def gen_field(self, target, field_name, field):
        """
        Generate value by field.

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        :param relation: Instance of :class:`Field`
        """
        column = field.scheme

        if (column.autoincrement and column.primary_key) \
                or column.nullable:
            return False

        if column.default:
            default = column.default.arg(target) \
                if column.default.is_callable \
                else column.default.arg
            return setattr(target, field_name, default)

        return super(TypeMixer, self).gen_value(target, field_name, column)

    def gen_random(self, target, field_name):
        """
        Generate random value of field with `field_name` for `target`

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        """
        field = self.fields.get(field_name)
        if isinstance(field, Relation):
            return self.gen_relation(target, field_name, field)
        return super(TypeMixer, self).gen_value(
            target, field_name, field.scheme, fake=False)

    def gen_select(self, target, field_name):
        """
        Select exists value from database.

        :param target: Target for generate value.
        :param field_name: Name of field for generation.
        """
        if not self.mixer or not self.mixer.session:
            return False

        relation = self.mapper.get_property(field_name)
        value = self.mixer.session.query(
            relation.mapper.class_
        ).order_by(func.random()).first()
        setattr(target, field_name, value)

    def gen_relation(self, target, field_name, relation):
        """
        Generate a related relation by `relation`

        :param target: Target for generate value.
        :param field_name: Name of relation for generation.
        :param relation: Instance of :class:`Relation`

        """
        rel = relation.scheme
        if rel.direction == MANYTOONE:
            col = rel.local_remote_pairs[0][0]
            if col.nullable and not relation.params:
                return False

            value = self.mixer and self.mixer.blend(
                rel.mapper.class_,
                **relation.params
            ) or TypeMixer(
                rel.mapper.class_,
                mixer=self.mixer,
                generator=self.generator,
                fake=self.fake,
            ).blend(**relation.params)

            setattr(target, rel.key, value)
            setattr(target, col.name,
                    rel.mapper.identity_key_from_instance(value)[1][0])

    def make_generator(self, column, field_name=None, fake=False):
        """ Make values generator for column.

            :param column: SqlAlchemy column
            :param field_name: Field name
            :param fake: Force fake data
        """
        kwargs = dict()
        ftype = type(column.type)
        stype = self.generator.cls_to_simple(ftype)

        if stype is str:
            kwargs['length'] = column.type.length

        return self.generator.gen_maker(stype, field_name, fake)(**kwargs)

    def __load_fields(self):
        """ Prepare SQLALchemyTypeMixer.
            Select columns and relations for data generation.
        """
        mapper = self.cls._sa_class_manager.mapper
        relations = set()

        for rel in mapper.relationships:
            relations |= rel.local_columns
            yield rel.key, Relation(rel, rel.key)

        for column in mapper.columns:
            if not column in relations:
                yield column.name, Field(column, column.name)


class Mixer(BaseMixer):

    type_mixer_cls = TypeMixer

    def __init__(self, session=None, commit=False, **params):
        """Initialize the SQLAlchemy Mixer.

        :param fake: (True) Generate fake data instead of random data.
        :param session: SQLAlchemy session. Using for commits.
        :param commit: (False) Commit instance to session after creation.

        """
        super(Mixer, self).__init__(**params)
        self.session = session
        assert not commit or self.session, "Set session for commits"
        self.commit = commit

    def post_generate(self, result, type_mixer):
        if self.commit:
            self.session.add(result)
            self.session.commit()

        return result

# lint_ignore=W0212



from __future__ import absolute_import

import datetime
import decimal
from collections import defaultdict

from django.db import models

from .. import generators as g, mix_types as t, six
from ..main import (
    Field, Relation,
    TypeMixerMeta as BaseTypeMixerMeta,
    TypeMixer as BaseTypeMixer,
    Generator as BaseGenerator,
    Mixer as BaseMixer)


class Generator(BaseGenerator):
    types = {
        (models.CharField, models.SlugField): str,
        models.TextField: t.Text,
        models.BooleanField: bool,
        models.BigIntegerField: t.BigInteger,
        (models.AutoField, models.IntegerField): int,
        models.PositiveIntegerField: t.PositiveInteger,
        models.PositiveSmallIntegerField: t.PositiveSmallInteger,
        models.SmallIntegerField: t.SmallInteger,
        models.DateField: datetime.date,
        models.DateTimeField: datetime.datetime,
        models.TimeField: datetime.time,
        models.DecimalField: decimal.Decimal,
        models.FloatField: float,
        models.EmailField: t.EmailString,
        models.IPAddressField: t.IP4String,
    }


class TypeMixerMeta(BaseTypeMixerMeta):

    def __load_cls(cls, cls_type):
        if isinstance(cls_type, six.string_types):
            assert '.' in cls_type, ("'model_class' must be either a model"
                                     " or a model name in the format"
                                     " app_label.model_name")
            app_label, model_name = cls_type.split(".")
            cls_type = models.get_model(app_label, model_name)
        return cls_type


class TypeMixer(six.with_metaclass(TypeMixerMeta, BaseTypeMixer)):

    __metaclass__ = TypeMixerMeta

    generator = Generator

    def blend(self, **values):
        self.post_save_values = defaultdict(list)
        return super(TypeMixer, self).blend(**values)

    def set_value(self, target, field_name, field_value):

        field = self.fields.get(field_name)
        if field and field.scheme in self.cls._meta.local_many_to_many:
            if not isinstance(field_value, (list, tuple)):
                field_value = [field_value]
            self.post_save_values[field_name] += field_value
            return False

        return super(TypeMixer, self).set_value(
            target, field_name, field_value
        )

    def gen_field(self, target, field_name, field):
        if field.scheme.null and field.scheme.blank:
            return None

        if field.scheme.has_default():
            return self.set_value(
                target, field_name, field.scheme.get_default())

        if field.scheme.choices:
            choice = g.get_choice([c[0] for c in field.scheme.choices])
            return self.set_value(target, field_name, choice)

        super(TypeMixer, self).gen_field(target, field_name, field)

    def gen_select(self, target, field_name):
        field = self.fields.get(field_name)
        if field.is_relation:
            try:
                return field.scheme.model.objects.order_by('?')[0]
            except Exception:
                raise Exception(
                    "Cannot find a value for the field: '{0}'".format(
                        field_name
                    ))
        return super(TypeMixer, self).gen_select(target, field_name)

    def gen_random(self, target, field_name):
        field = self.fields.get(field_name)
        if field.is_relation:
            return self.gen_relation(target, field_name, field)
        return super(TypeMixer, self).gen_random(target, field_name)

    def gen_relation(self, target, field_name, relation):
        if relation.scheme.null or relation.scheme.blank:
            return None

        rel = relation.scheme
        new_scheme = rel.related.parent_model

        if new_scheme == self.cls:
            if relation.scheme in self.cls._meta.local_many_to_many:
                self.post_save_values[field_name] = [target]
            else:
                self.gen_select(target, field_name)
            return False

        value = self.mixer and self.mixer.blend(
            new_scheme, **relation.params
        ) or TypeMixer(
            new_scheme, mixer=self.mixer, generator=self.generator,
            fake=self.fake,
        ).blend(**relation.params)

        self.set_value(target, rel.name, value)

    def make_generator(self, field, fname=None, fake=False):
        fcls = type(field)
        stype = self.generator.cls_to_simple(fcls)
        gen_maker = self.generator.gen_maker(fcls, fname, fake)
        kwargs = dict()

        if stype is str:
            kwargs['length'] = field.max_length

        elif stype is decimal.Decimal:
            kwargs['i'] = field.max_digits - field.decimal_places
            kwargs['d'] = field.decimal_places

        if fcls is models.CommaSeparatedIntegerField:
            return g.gen_choices([1, 2, 3, 4, 5, 6, 7, 8, 9, 0],
                                 field.max_length)

        return gen_maker(**kwargs)

    def __load_fields(self):
        for field in self.cls._meta.fields:

            if isinstance(field, models.AutoField)\
                    and self.mixer and self.mixer.commit:
                continue

            if isinstance(field, models.ForeignKey):
                yield field.name, Relation(field, field.name)
                continue

            yield field.name, Field(field, field.name)

        for field in self.cls._meta.local_many_to_many:
            yield field.name, Relation(field, field.name)


class Mixer(BaseMixer):

    type_mixer_cls = TypeMixer

    def __init__(self, commit=True, **params):
        super(Mixer, self).__init__(**params)
        self.commit = commit

    def post_generate(self, result, type_mixer):
        if self.commit:
            result.save()

        return result


# lint_ignore=W0212,W0201,E1002,F0401



class BigInteger:
    pass


class EmailString:
    pass


class HostnameString:
    pass


class IP4String:
    pass


class NullOrBoolean:
    pass


class PositiveDecimal:
    pass


class PositiveInteger:
    pass


class PositiveSmallInteger:
    pass


class SmallInteger:
    pass


class Text:
    pass



#!/usr/bin/env python

"""
mixer
-----

mixer -- Description

"""

import sys
from os import path as op

from setuptools import setup

from mixer import __version__, __project__, __license__


def read(fname):
    try:
        return open(op.join(op.dirname(__file__), fname)).read()
    except IOError:
        return ''

tests_require = [
    'django',
    'flask-sqlalchemy',
    'sqlalchemy',
]
install_requires = [l for l in read('requirements.txt').split('\n')
                    if l and not l.startswith('#')]

if sys.version_info < (2, 7):
    install_requires.append('importlib')

elif sys.version_info > (3, 0):
    tests_require.remove('flask-sqlalchemy')


setup(
    name=__project__,
    version=__version__,
    license=__license__,
    description=read('DESCRIPTION'),
    long_description=read('README.rst'),
    platforms=('Any'),

    author='Kirill Klenov',
    author_email='horneds@gmail.com',
    url='http://github.com/klen/mixer',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Natural Language :: Russian',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python',
    ],

    py_modules=['mixer'],
    install_requires=install_requires,
    tests_require=tests_require,
    test_suite='tests',
)

# lint_ignore=F0401
